'' =================================================================================================
''
''   File....... jd_animation__2017-08-18.spin
''   Purpose.... Jeff David Animation Demo
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (C) 2011-2017 Jon McPhalen
''   E-mail..... 
''   Started.... 
''   Updated.... 18 AUG 2017
''
'' =================================================================================================


con

  ' Critical: The revision letter must match the PCB that will receive this program. There were
  '           several design updates to the Rev F PCB.

  PCB_REV = "E"


con { timing }

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = _xinfreq * (_clkmode >> 6)                         ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us
                                                                 
  EFX_SLOW =   2_400                                            ' EFX-TEK baud rates                                             
  EFX_FAST =  38_400                                              
                                                                 
  TRM_BAUD = 115_200                                            ' terminal baud rate
                                                                 
                                                                 
con { io pins }                                                  
                                                                 
  RX1     = 31  { I }                                           ' serial / programming
  TX1     = 30  { O }                                            
                                                                 
  SDA1    = 29  { I/O }                                         ' i2c / eeprom
  SCL1    = 28  { I/O }                                          

  XP0     = 27  { I/O }                                         ' extra port 0 - not on Rev E or lower
  XP1     = 24  { I/O }                                         ' extra port 1 - not on Rev E or lower 
  XP2     = 19  { I/O }                                         ' extra port 2 - not on Rev E or lower    
  XP3     = 16  { I/O }                                         ' extra port 3 - not on Rev E or lower 
                                                         
  OUT_EN  = 27  { O }                                           ' outputs enable pin (low)     
                                                                 
  SIO     = 26  { I/O }                                         ' TTL serial io
                                                              
  SIO_RX  = 26  { I/O }                                         ' TTL serial io
  SIO_TX  = 25  { I/O }                                         ' TTL serial io (not enabled)  
                                                             
  RX2     = 23  { I }                                           ' RS-485 port
  RX2_EN  = 22  { O }                                                 
  TX2_EN  = 21  { O }                                                 
  TX2     = 20  { O }

' R_LED   = 18  { O }                                           ' bi-color LED (Rev F)
' G_LED   = 17  { O }
  
  G_LED   = 18  { O }                                           ' bi-color LED (Rev E)
  R_LED   = 17  { O }                                                 
                                                             
  OUT7    = 15  { O }                                           ' outputs
  OUT6    = 14  { O }                                                 
  OUT5    = 13  { O }                                                 
  OUT4    = 12  { O }                                                 
  OUT3    = 11  { O }                                                 
  OUT2    = 10  { O }                                                 
  OUT1    =  9  { O }                                                 
  OUT0    =  8  { O }                                                 
                                                                 
  OPT_BR  =  7  { I/O }                                         ' options switches
  OPT_A1  =  6  { I/O }                                                  
  OPT_A0  =  5  { I/O }                                                  
  OPT_SM  =  4  { I/O }                                                  
                                                                 
  DMX_A8  =  3  { do not use with Rev C and earlier PCBs }      ' high-bit of DMX address
                                                                 
  DO_165  =  2  { I }                                           ' control lines for 74x165
  CLK_165 =  1  { O }                                                  
  LD_165  =  0  { O }                                                  


  ' for EFX-TEK GameOn adapter module (#99001)

  WII_DAT =  OPT_A0
  WII_CLK =  OPT_SM 

  
  ' for EFX-TEK uSD adapter module (#99002)
  ' -- must open all option switches when installed

  SD_CS   =  OPT_BR  '   7                                  
  SD_DI   =  OPT_A1  '   6
  SD_CLK  =  OPT_A0  '   5
  SD_DO   =  OPT_SM  '   4                                         
                                                                 
                                                                 
con

  CHANNELS = 5                                                          
                                                                 
  #true, ON, OFF                                                ' digital control
                                                                 
  #0, NO, YES                                                    
                                                                 
  #0, BLK, RED, GRN, YEL                                        ' for r/g led
                                                                 
  #0, NOCON, RX, TX, RXTX                                       ' for rs-485
                                                                 
  #0, LSBFIRST, MSBFIRST                                         

  DEBUG = YES                                                   ' use terminal for testing

  AUDIO_ON = NO                                                 ' disable AP-16+ connection

                                                            
obj                              
                                 
' main                                                          ' * master Spin cog                   
  time       : "jm_time_80"                                     '   timing and delays (80MHz system) 
  prng       : "jm_prng"                                        '   pseudo-random number generator     
  io         : "jm_io"                                          '   essential io
  servo      : "jm_servo8"                                      ' * servo driver 
  outs       : "jm_pwm8"                                        ' * pwm driver for outputs
  term       : "jm_fullduplexserial"                            ' * serial IO for terminal
  audio      : "jm_fullduplexserial"                            ' * serial IO for AP-16+
  sd         : "fsrw"                                           ' * uSD file IO
' background                                                    ' * background RGY LED control, inputs scanning 
  
' * uses cog when loaded
                                 
                                 
var

  ' do not change order or type!!!

  byte  pos1
  byte  pos2
  byte  pos3
  byte  loops


dat

  ServoFile     byte    "servos.txt", 0                                                           
                                                                 
                                                      
pub main | check, t, line, idx                                              
                                                                 
  setup                                                         ' setup io and objects

  ' 1) Downlaod program with F10
  ' 2) Open PST with F12
  ' 3) Set PST baud to 115200, com port, then Enable
  ' 4) Press [Enter] on keyboard

  term.rxflush
  term.rx
  term.tx(term#CLS)
  term.str(string("Data read & parse demo", 13))

  check := open_file(@ServoFile, "r")                           ' try to open file
  
  if (check == false)
    term.str(string("-- Error: Cannot open servo file.", 13))

  else
    line := 1
    time.set(0)                                                 ' reset show timer  
    t := cnt                                                    ' sync loop timer
    repeat
      time.mark                                                 ' update show timer
      check := rd_line(@linebuf, 80)
      if (check =< 0)
        quit
      else
        replace(@linebuf, 13, 0)                                ' strip CR from input line
        term.tx(13)  
        term.rjdec(line++, 4, "0")
        term.txn(" ", 2)
        term.str(@linebuf)                                      ' show raw input
        parse_values(@linebuf)                                  ' parse data from it 
        term.tx(term#CRSR_X)
        term.tx(26)
        repeat idx from 0 to 3                                  ' display parsed values
          term.rjdec(pos1[idx], 5, " ")
        waitcnt(t += (loops * MS_001 * 20))                     ' real-time delay

    check := time.millis
    term.str(string(13, 13, "Show time = "))
    term.dec(time.millis / 1000)
    term.tx(".")
    term.rjdec(time.millis // 1000, 3, "0")
    term.str(string(" seconds"))
                                                       
  repeat                                                         
    waitcnt(0)


pub replace(p_str, char, fix)

'' Replace instances of char with fix inside string at p_str

  repeat strsize(p_str)
    if (byte[p_str] == char)
      byte[p_str] := fix
    ++p_str

    
pub parse_values(p_str) | idx

'' Extract four byte-wide values from data file

  repeat idx from 0 to 3
    pos1[idx] := extract_dec(field(idx, p_str, ","))


pub extract_dec(p_str) : value | c

'' Extract decimal value from string at p_str
'' -- stops on encountering non-digit character

  repeat
    c := byte[p_str++]
    if ((c => "0") and (c =< "9"))
      value *= 10
      value += c - "0"
    else
      quit


pub field(fn, p_str, sep) | c

'' Return pointer to desired field in string
'' -- fn is field number (0..n)
'' -- p_str is pointer to string to search
'' -- sep is field separator character 

  if (fn > ccount(p_str, sep))
    return -1                                                   ' error

  if (fn > 0)
    repeat
      if (byte[p_str++] == sep)
        if (--fn == 0)
          quit   

  return p_str
   

pub ccount(p_str, c) : n

'' Counts the occurances of c in string at p_str

  repeat strsize(p_str)
    if (byte[p_str++] == c)
      ++n

                                                           
pub setup
                                                                 
'' Configure HC-8+ IO and support objects                        
                                                                 
  time.start                                                    ' setup for timing/delays
                                                                 
  prng.seed(-cnt, -cnt ~> 2, $CAFE_BABE, cnt <- 2, cnt)         ' seed randomizer                             
                                                                 
  io.start(0, 0)                                                ' clear pins (this cog)

  if (PCB_REV =< "E")                                  
    io.low(OUT_EN)                                              ' enable output driver
                                                               
  outs.start(CHANNELS, OUT0)                                    ' start outputs driver *
                                                                 
  set_rs485(NOCON)                                              ' RS-485 not connected

  if (DEBUG == YES)
    term.start(RX1, TX1, %0000, TRM_BAUD)                       ' start serial for terminal *

  if (AUDIO_ON == YES)
    ap16_setup(SIO, 38_400)                                     ' start serial for AP-16+ *   

  mount_sd                                                      ' start sd access *
                                                    
  bcog := cognew(background, @bcstack) + 1                      ' start background cog *


con

  { --------------------------------- }
  {  S U P P O R T   R O U T I N E S  } 
  { --------------------------------- }

                                                           
pub ttl_pin | check                                                    
                                                                 
'' Reads TTL states for 1-of-8 input                             
'' -- returns -1 (no input)                                      
'' -- returns 0 to 7 for input on IN0 to IN7                     
''    * only one input allowed

  check := ttlpins                                  
                                                                 
  if (bit_count(check) == 1)                                    ' only one button pressed?
    return bit_pos(check, 0)                                    '  yes, return button #
  else                                                           
    return -1                                                   '  none or bad input
                                                                 

pub ttl_inputs 

'' Return TTL inputs status
'' -- pins are scanned by background cog

  return ttlpins


pub dmx_address 

'' Return DMX address switch setting
'' -- switches are scanned by background cog
'' -- Note: For Rev C and lower boards, do not use DMX 256 switch -- leave it OFF

  return dmxaddr


pub read_xport                                                  ' Rev F and higher only
                                                                 
'' Returns state of xport bits
'' -- does not set dira bits to 0
'' -- pins are not contiguous!  

  if (PCB_REV < "F")
    return %0000
  else
    return (ina[XP3] << 3) | (ina[XP2] << 2) | (ina[XP1] << 1) | ina[XP0]


pub write_xport(value, outsmask)                                ' Rev F and higher only    
                                                                 
'' Write value to xport pins                                     
'' -- outsmask used to set xport output pins                     
'' -- will not change pins with 0 bit in outsmask                
                                                                 
  if (PCB_REV < "F")                                             
    return

  ' unrolled
  ' -- pins are not contiguous!

  if (outsmask & %1000)
    io.set(XP3, value >> 3)  

  if (outsmask & %0100)
    io.set(XP2, value >> 2)

  if (outsmask & %0010)
    io.set(XP1, value >> 1)    

  if (outsmask & %0001)
    io.set(XP0, value >> 0)

    
pub set_rgy_solid(color)                                              
                                                                  
'' Sets R/G LED                                                   
'' -- direct control only                                         
'' -- red/green/yellow + blinking requires support cog   
                                                                  
  ifnot (bcog)                                                  ' if support cog not running
    dira[R_LED..G_LED] := %11                                   ' make outputs
    case color                                                  ' set color to state                  
      OFF : outa[R_LED..G_LED] := %00                                                                 
      RED : outa[R_LED..G_LED] := %10                                                                 
      GRN : outa[R_LED..G_LED] := %01                                                                 
      YEL : outa[R_LED..G_LED] := %00                           ' yellow requires active driver       

  else                                                           
    case color                                                   
      OFF : set_rgy_flash(OFF, 100, OFF, 100)                   ' set to solid color 
      RED : set_rgy_flash(RED, 100, RED, 100)                          
      GRN : set_rgy_flash(GRN, 100, GRN, 100)                           
      YEL : set_rgy_flash(YEL, 100, YEL, 100)


pub set_rgy_flash(p0c, p0ms, p1c, p1ms)                                 
                                                                  
'' Explicit (control registers) setup of OGRY LED
'' -- requires support cog to be running               
'' -- p0c is phase 0 color (OFF, GRN, RED, YEL)                   
'' -- p0ms is phase 0 timing in milliseconds                      
'' -- p1c is phase 1 color                                        
'' -- p1ms is phase 1 timing in milliseconds

  ifnot (bcog)
    return                     
                                                                  
  if ((p0c => OFF) and (p0c =< YEL))                            ' phase 0 color
    rgycolor[0] := p0c                                           
  else                                                           
    rgycolor[0] := OFF                                           
                                                                 
  if (p0ms > 0)                                                 ' phase 0 timing                                           
    rgytime[0] := p0ms                                           
  else                                                           
    rgytime[0] := 100                                            
                                                                 
  if ((p1c => OFF) and (p1c =< YEL))                            ' phase 1 color                             
    rgycolor[1] := p1c                                           
  else                                                           
    rgycolor[1] := OFF                                           
                                                                 
  if (p1ms > 0)                                                 ' phase 1 timing                                              
    rgytime[1] := p1ms                                           
  else                                                            
    rgytime[1] := 100                                             

                                                           
pub set_rs485(state)                                              
                                                                  
'' Sets RS-485 RX and TX enable inputs                            
                                                                  
  case state                                                      
    NOCON : outa[RX2_EN..TX2_EN] := %10                           
    RX    : outa[RX2_EN..TX2_EN] := %00                           
    TX    : outa[RX2_EN..TX2_EN] := %11                           
    RXTX  : outa[RX2_EN..TX2_EN] := %01                           
                                                                  
  dira[RX2_EN..TX2_EN] := %11                                     
                                                                  
                                                                  
pub bit_count(value) : bc                                        
                                                                  
'' Returns # of bits set (1) in value                             
                                                                  
  repeat while (value)                                          ' loop while there are 1s
    bc += (value & %1)                                          '  add bit value to count
    value >>= 1                                                 '  next bit
                                                                 
                                                                 
pub bit_pos(value, mode)                                         
                                                                 
'' Returns position of 1st bit set (0..31)                       
'' -- mode 0 to scan from lsb, mode 1 to scan from msb           
'' -- -1 = no bits set                                           
                                                                 
  if (value == 0)                                               ' if no bits                                   
    return -1                                                   '  return -1
  else                                                           
    if (mode == LSBFIRST)                                       ' check from LSB
      value ><= 32                                              '  flip for >|
      return (32 - >|value)                                      
    else                                                          
      return (>|value - 1)                                        
                                                                  
                                                                  
pub bit_val(value, pos)                                           
                                                            
'' Returns bit (0..1) from pos (0..31) in value                   
                                                                  
  if ((pos => 0) and (pos =< 31))                                 
    return (value >> pos) & 1                                     
  else                                                            
    return 0


con

  { ------------------------------- }
  {  A P - 1 6 +   R O U T I N E S  } 
  { ------------------------------- }


dat

  Header                byte    "!AP16", 0
  Version               byte    "???", 0
 

pub ap16_setup(pin, baud)

'' Start audio serial driver on pin at baud (half-duplex true mode)

  audio.start(pin, pin, %1100, baud)
  time.pause(10)
  audio.rxflush


pub ap16_version(addr) | idx, c

'' Returns version string from AP-16+ at addr

  audio.str(@Header)
  audio.tx(addr)
  audio.tx("V")
  audio.rxflush

  repeat idx from 0 to 2
    c := audio.rxtime(25)
    if (c => 0)
      byte[@Version][idx] := c
    else
      byte[@Version][idx] := "?"

  return @Version  
 

pub ap16_status(addr)

'' Returns AP-16+ status byte

  audio.str(@Header)
  audio.tx(addr)
  audio.tx("G")
  audio.rxflush

  return audio.rxtime(25)


pub ap16_playing(addr) | check

'' Returns playing status from AP-16+ at addr
'' -- returns 1 (playing), 0 (not playing), or -1 (no response)

  check := ap16_status(addr) 

  if (check => 0)
    return (check >> 7) & %1                                    ' return status bit (bit7)
  else
    return -1                                                   ' no response
 
    
pub ap16_sfx(addr, sfx, rpts)

'' Plays SFX file on AP-16+ at addr

  audio.str(@Header)
  audio.tx(addr)
  audio.str(string("PS"))
  audio.tx(sfx)
  audio.tx(rpts)   


pub ap16_aux(addr, aux, rpts)

'' Plays AUX file on AP-16+ at addr

  audio.str(@Header)
  audio.tx(addr)
  audio.str(string("PA"))
  audio.tx(aux)
  audio.tx(rpts)


pub ap16_wav(addr, p_str, rpts)

'' Plays WAV file (name at p_str) on AP-16+ at addr

  audio.str(@Header)
  audio.tx(addr)
  audio.str(string("PW"))
  audio.str(p_str)
  audio.tx(13)      
  audio.tx(rpts)
   

pub ap16_stop(addr)

'' Stops AP-16+ at addr
'' -- use $FF for all AP-16+s connected

  audio.str(@Header)
  audio.tx(addr)
  audio.tx("X")  


con                                                              
                                                                 
  { ----------------- }                                          
  {  F I L E   I / O  }                                          
  { ----------------- }                                          
                                                                 
                                                                 
var                                                              
                                                                 
  long  mounted                                                  
                                                                 
  byte  filename[8 + 1 + 3 + 1]                                 ' 8.3 plus 0
  byte  linebuf[80 + 1]                                         ' line buffer
                                                                 
                                                                 
pub mount_sd | check                                             
                                                                 
'' Attempt to mount SD card                                      
'' -- returns true if SD card successfully mounted

  check := \sd.mount_explicit(SD_DO, SD_CLK, SD_DI, SD_CS)

  mounted := (check == 0)   

  return mounted


pub dismount_sd

'' Unmounts SD card

  \sd.unmount
  mounted := false


pub has_file(p_str) | check

'' Returns true if file is on SD card

  check := \sd.popen(p_str, "r")
  if (check == 0)
    return true
  else
    return false 


pub open_file(p_str, mode) | check

'' Attempt to open file
'' -- p_str is pointer to filename (z-string)
'' -- mode is [lower-case] single character, (e.g., "r" for read)
'' -- sd card must be mounted before open

  if (mounted)                                                   
    check := \sd.popen(p_str, mode)                             ' attempt to open
    if (check == 0)                                              
      if (mode == "r")                                           
        \sd.seek(0)                                             ' force sector read 
      return true                                                
    else                                                         
      return false
  else
    return false


pub close_file

'' Closes open file

  \sd.pclose


pub rd_line(p_str, n) | c, len

'' Reads line of text from open text file
'' -- terminated by LF or EOF
'' -- p_str is address of string buffer
'' -- n is maximum number of characters to read from line

  len := 0                                                      ' index into string
                                                                 
  repeat n                                                       
    c := \sd.pgetc                                              ' get a character
    if (c < 0)                                                  ' end of file
      if (len == 0)                                             ' if nothing
        return -1                                               ' return empty
      else                                                       
        quit                                                     
                                                                 
    if (c <> 10)                                                ' if not LF char
      byte[p_str++] := c                                        '  add to buffer
      len++                                                      
    else                                                        ' if LF
      quit                                                      '  we're done
                                                                 
  byte[p_str] := 0                                              ' terminate end of line
                                                                 
  return len
  

pub wr_line(p_str, usecr, uself)

'' Writes z-string to open text file
'' -- set usecr to true to add CR
'' -- set uself to true to add LF

  \sd.pputs(p_str)
  if (usecr)
    \sd.pputc(13)
  if (uself)
    \sd.pputc(10)


pub rd_buf(p_buf, n) | check

'' Reads n bytes from file to buffer at p_buf
'' -- returns number of bytes read

  return \sd.pread(p_buf, n)


pub wr_buf(p_buf, n) | check   

'' Writes n bytes to file from buffer at p_buf
'' -- returns number of bytes written

  return \sd.pwrite(p_buf, n)


pub directory | fcount, check, idx, p_fname, fsize

'' Show directory (root only) of uSD card

  sd.opendir                                                    ' setup for .nextfile
  fcount := 0                                                   ' reset files count
                                                                 
  term.str(string("Directory", term#CR, term#CR))                
                                                                 
  repeat                                                         
    bytefill(@filename, 0, 13)                                   
    check := \sd.nextfile(@filename)                            ' get next file in root
    if (check == 0)                                             ' if found
      fcount += 1                                               '  increment file count
      term.tx(" ")
      term.str(@filename)                                       '  show name
      term.tx(13)                                           
    else                                                        ' else
      quit                                                      '  done
                                                                 
  if (fcount > 0)                                                
    term.tx(13)
    term.dec(fcount)
    term.str(string(" file(s) found."))
  else
    term.str(string("No files found."))

  sd.pclose


pub show_file(p_file) | len

'' Display contents of text file

  ifnot (mounted)
    term.str(string("SD not mounted.", 13))
    return

  if (open_file(p_file, "r"))
    repeat
      bytefill(@linebuf, 0, 81)
      len := rd_line(@linebuf, 80)
      if (len > 0)
        term.str(@linebuf)
        time.pause(5)
      else
        close_file
        quit 


con

  { ----------------------------- }
  {  B A C K G R O U N D   C O G  }
  {  - global milliseconds        }
  {  - R/G LED                    }
  {  - TTL/DMX inputs scanning    }
  { ----------------------------- }
  

var  

  long  bcog                                                    ' background cog #
  long  bcstack[32]                                             ' stack for background cog
                                                                 
  long  millis                                                  ' global milliseconds register
                                                                 
                                                                 
pri background | t                                              ' launch with cognew()
                                                                 
  io.low(R_LED)                                                 ' setup R/G LED pins
  io.low(G_LED)                                                  
                                                                 
  io.high(LD_165)                                               ' setup x165 io pins
  io.low(CLK_165)                                                
  io.input(DO_165)                                               
                                                                 
  millis := 0                                                    
                                                                 
  t := cnt                                                      ' sync loop timer
  repeat                                                         
    waitcnt(t += MS_001)                                        ' run loop every millisecond
    ++millis                                                     
    refresh_rg_led                                              ' update the R/G LED
    scan_ttl_ins                                                ' re-scan TTL & DMX inputs
                                                                 
                                                                 
var                                                              
                                                                 
  long  rgycolor[2]                                             ' led phase colors
  long  rgytime[2]                                              ' led phase timing (ms)
  long  rgphase                                                 ' current phase (red or green chip)
  long  phasetimer                                              ' time in phase
  long  rgtimer                                                 ' timer for rg process
                                                                 
                                                                 
pri refresh_rg_led                                              ' call only from support()
                                                                 
  if (++phasetimer => rgytime[rgphase])                         ' done with this phase?
    phasetimer := 0                                             ' yes, reset timer
    rgphase := 1 - rgphase                                      '  and invert phase
                                                                 
  if (++rgtimer == 16)                                           
    rgtimer := 0                                                 
                                                                 
  case rgycolor[rgphase]                                        ' set led to color for phase
    OFF:                                                         
      outa[R_LED..G_LED] := %00                                  
                                                                 
    GRN:
      outa[R_LED..G_LED] := %01  
                                                                 
    RED:
      outa[R_LED..G_LED] := %10 
                                                                   
    YEL:
      if (rgtimer < 2)                                                                       
        outa[R_LED..G_LED] := %10                               ' 1 red                        
      else                                                                                   
        outa[R_LED..G_LED] := %01                               ' 15                  
                                                          

var

  long  ttlpins
  long  dmxaddr                                                  

                                                                
pri scan_ttl_ins | tempin                                         
                                                                 
'' Scan TTL and DMX address inputs                               
                                                                 
  outa[LD_165] := 0                                             ' blip Shift/Load line    
  outa[LD_165] := 1                                              

  ' unrolled for best speed

  tempin := (tempin << 1) | ina[DO_165]                         ' bit15
  outa[CLK_165] := 1                                            ' blip clock  
  outa[CLK_165] := 0                                                        
  tempin := (tempin << 1) | ina[DO_165]
  outa[CLK_165] := 1                   
  outa[CLK_165] := 0                   
  tempin := (tempin << 1) | ina[DO_165]
  outa[CLK_165] := 1                   
  outa[CLK_165] := 0                   
  tempin := (tempin << 1) | ina[DO_165]
  outa[CLK_165] := 1                   
  outa[CLK_165] := 0
  tempin := (tempin << 1) | ina[DO_165] 
  outa[CLK_165] := 1                    
  outa[CLK_165] := 0                    
  tempin := (tempin << 1) | ina[DO_165] 
  outa[CLK_165] := 1                    
  outa[CLK_165] := 0                    
  tempin := (tempin << 1) | ina[DO_165] 
  outa[CLK_165] := 1                    
  outa[CLK_165] := 0
  tempin := (tempin << 1) | ina[DO_165] 
  outa[CLK_165] := 1                    
  outa[CLK_165] := 0
  
  tempin := (tempin << 1) | ina[DO_165]                         ' bit7
  outa[CLK_165] := 1                    
  outa[CLK_165] := 0                    
  tempin := (tempin << 1) | ina[DO_165] 
  outa[CLK_165] := 1                    
  outa[CLK_165] := 0                    
  tempin := (tempin << 1) | ina[DO_165] 
  outa[CLK_165] := 1                    
  outa[CLK_165] := 0                    
  tempin := (tempin << 1) | ina[DO_165] 
  outa[CLK_165] := 1                    
  outa[CLK_165] := 0
  tempin := (tempin << 1) | ina[DO_165] 
  outa[CLK_165] := 1                    
  outa[CLK_165] := 0                    
  tempin := (tempin << 1) | ina[DO_165] 
  outa[CLK_165] := 1                    
  outa[CLK_165] := 0                    
  tempin := (tempin << 1) | ina[DO_165] 
  outa[CLK_165] := 1                    
  outa[CLK_165] := 0                                       
  tempin := (tempin << 1) | ina[DO_165]                         ' bit0                    
                                                         
  ttlpins := tempin.byte[0]                                     ' update global vars
  dmxaddr := tempin.byte[1] | (ina[DMX_A8] << 8)     

                                                 
dat { license }                                                  

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

}}      